/* 
  NMDCHub
  
    About: Protocol library for NMDC, the successor to DCBot, DCBot2 and mappyDC.
    Usage: Instantiate NMDCHub and poll NextEvent(), and compare it's type with
           NMDCEventTypes to see what happened.
  Depends: STL::stack, STL::map, STL::string and CSimpleSocket.
       
*/

#include "stdafx.h"
#include "ActiveSocket.h"
#include <string>
#include <queue>
#include <map>

// Buffer sizes
///////////////

#define BUFL 4096        /* Large buffer size (messages and chat)	*/
#define BUFS 256         /* Short buffer size (user nicks)			*/

// NMDCEvent
////////////

enum NMDCEventTypes {
	NMDC_NONE,           /* Nothing to report.       data1: -             data2: -         */
	NMDC_DEBUG_INFO,     /* Debugging information.   data1: undefined     data2: undefined */
	NMDC_PUBLIC_CHAT,    /* A public chat message.   data1: sender nick   data2: message   */
	NMDC_PRIVATE_MSG,    /* A private chat message.  data1: sender nick   data2: message   */
	NMDC_SYSTEM_MSG,     /* A system message.        data1: -             data2: message   */
	NMDC_USER_JOIN,      /* A user joined the hub.   data1: user nick     data2: -         */
	NMDC_USER_PART,      /* A user left the hub.     data1: user nick     data2: -         */
	NMDC_USER_UPDATE,    /* A user updated details.  data1: user nick     data2: -         */
	NMDC_NEED_PASS		 /* Send a password before doing anything else.                    */
};

typedef struct {
	int type;			 /* Compare with NMDCEventTypes				*/
	char data1[BUFS];    /* Type-dependent data (small buffer)		*/
	char data2[BUFL];    /* Type-dependent data (large buffer)		*/
} NMDCEvent;

// NMDCUser
///////////

typedef struct {
	char   szNick[BUFS];
	char   szDescription[BUFS];
	uint64 iShareSize;
} NMDCUser;

// NMDCHub
//////////

typedef std::map<std::string, NMDCUser> UMAP;

class NMDCHub {
private:
	// Variables
	NMDCUser self;
	CActiveSocket sock;
	bool bSockInitialised;
	bool bUsedPassword;
	UMAP userlist;

	// Construction & Deconstruction
	void Initialise(char* szAddress, int iPort, char* szNick, uint64 iShareSize);

	// Event handling
	std::queue<NMDCEvent> events;
	void push_event(int iType, char* data1, char* data2);

	// Protocol handlers
	void handle_command(char* szCommand);
	void process_messages();

	// Protocol helpers
	void      joinUser(NMDCUser usr);
	void      joinUser(char* szNick);
	NMDCUser  parse_myinfo(char* myinfo);
	NMDCEvent parse_to(char* to);
	void      parse_lock(char *lock, char* destBuff, int out_buffsize);
	void      parse_nicklist(char* list);

public:
	// Construction & Deconstruction
	NMDCHub(char* szAddress, int iPort, char* szNick, uint64 iShareSize);
	NMDCHub(char* szAddress, char* szNick, uint64 iShareSize); //will detect port in address
	void Disconnect();

	// Public methods
	bool Connected();
	void Login(char* szPassword);
	void SayPublic(char* szMessage);
	void SayPrivate(char* szUsername, char* szMessage);
	void SayExplicit(char* szMessage);
	
	// Event handling
	NMDCEvent NextEvent();

	// User handling
	int  GetUserCount();
	void GetUserNicks(NMDCUser* arrayDest, int maxcount);
	bool GetUserInfo(char* szUsername, NMDCUser* out);
};